% * Copyright (c) 2014, Andrew Kae, UMass Amherst
% * All rights reserved.
% *
% * Redistribution and use in source and binary forms, with or without
% * modification, are permitted provided that the following conditions are met:
% *     * Redistributions of source code must retain the above copyright
% *       notice, this list of conditions and the following disclaimer.
% *     * Redistributions in binary form must reproduce the above copyright
% *       notice, this list of conditions and the following disclaimer in the
% *       documentation and/or other materials provided with the distribution.
% *     * Neither the name of the author nor the organization may be used to 
% *       endorse or promote products derived from this software without specific 
% *       prior written permission.
% *
% * THIS SOFTWARE IS PROVIDED BY Andrew Kae ``AS IS'' AND ANY
% * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% * DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
% * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
% * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
% * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
% * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

% Code used in the following papers : 
% 
% * Gary B. Huang, Andrew Kae, Carl Doersch, and Erik Learned-Miller
%   Bounding the Probability of Error for High Precision Optical Character Recognition.
%   Journal of Machine Learning Research (JMLR), 2012. 
%
% * Andrew Kae, Gary Huang, Carl Doersch, and Erik Learned-Miller
%   Improving State-of-the-Art OCR through High-Precision Document-Specific Modeling
%   Proceedings of the IEEE Conference on Computer Vision and Pattern Recognition (CVPR), 2010.

%this function does alignment using SIFT
function process_SVM_align_SIFT(SVM_train, SVM_test, page_images, train_file, test_file, Tess_file, ...
    avg_height, avg_width, mean_dps)
    
    %SIFT options
    %note that this is for both rows and cols for a total of num_spatial_bins^2 bins
    num_spatial_bins = 4;

    num_orient_bins = 8;
    
    %set the scale factor so that it takes up the whole image
    sigma = 1;

    %determine the appropriate magnification for this task
    magnif = ceil(max([avg_height avg_width]) / num_spatial_bins);                        
    
    % I've noticed empirically having a magnification smaller than 3 does not help    
    %make sure magnif >= 3
    magnif = max(magnif, 3);
    
    %try aligning each test patch to both mean_patch1 and mean_patch2 using the distance to the SIFT
    %descriptor to guide the alignment.  Then use the best alignment as the SIFT descriptor    
            
    %first compute the dp for the mean patches
    metric = 'L2';           
    
    %print out the data
           
    %print out the training data
    train_fid = fopen(train_file, 'w');        
    
    %print the positive examples
    for i=1:length(SVM_train(:, 1))
        label = SVM_train(i,1);
        
        patch = page_images{SVM_train(i,2)}(SVM_train(i,5):SVM_train(i,6), SVM_train(i,7):SVM_train(i,8));        
            
        [patch_height, patch_width] = size(patch);

        %find the best "alignment" using SIFT
        [min_dist, dp] = sift_search(patch, mean_dps(label, :)', 'L2', magnif, sigma, num_spatial_bins ,...
            num_orient_bins);
                
        write_svm(train_fid, dp', label);
    end
    
    fclose(train_fid);
    
    %print the test examples
    gold_fid = fopen(Tess_file, 'w');
    test_fid = fopen(test_file, 'w');

    for i=1:length(SVM_test(:, 1))
        gold_label = SVM_test(i,1);
        
        %gold first, tesseract second
        fprintf(gold_fid, '%u %u\n', gold_label, SVM_test(i,2));
        
        patch = page_images{SVM_test(i,3)}(SVM_test(i,6):SVM_test(i,7), SVM_test(i,8):SVM_test(i,9));        
            
        [patch_height, patch_width] = size(patch);

        dp_opt = [];
        min_seen = inf;
        
        %search through all the mean dps to find the best "alignment" using SIFT
        for j=1:size(mean_dps, 1)            
            [min_dist, dp] = sift_search(patch, mean_dps(j, :)', 'L2', magnif, sigma, num_spatial_bins ,...
                num_orient_bins);
            
            if (min_dist < min_seen)
                min_seen = min_dist;
                dp_opt = dp;
            end
        end
        
        write_svm(test_fid, dp_opt', gold_label);
    end
   
    fclose(gold_fid);
    fclose(test_fid);    
end
